%module bonjour
%{
/* Includes the header in the wrapper code */
#include <dns_sd.h>

DNSServiceRef * AllocateDNSServiceRef()
{
    DNSServiceRef * ref = (DNSServiceRef*)malloc(sizeof(DNSServiceRef));
    return ref;
}

%}

%include typemaps.i
typedef unsigned short int uint16_t;
typedef long int32_t;
typedef unsigned int uint32_t;
typedef unsigned long long uint64_t;
typedef uint32_t DNSServiceFlags;
typedef int32_t DNSServiceErrorType;

%inline %{


#if defined(_WIN32)
#define DNSSD_API __stdcall
#else
#define DNSSD_API
#endif

typedef struct {
    void *func;
    void *userdata;
    } CallbackStruct;

static void DNSSD_API
service_register_callback
    (
    DNSServiceRef                       sdRef,
    DNSServiceFlags                     flags,
    DNSServiceErrorType                 errorCode,
    const char                          *name,
    const char                          *regtype,
    const char                          *domain,
    void                                *context
    ) 
    {
    PyObject *func ,*userdata, *arglist, *result;
    
    CallbackStruct *cbStruct = (CallbackStruct *)context;
    func = (PyObject*)cbStruct->func;
    userdata = (PyObject*)cbStruct->userdata;

    #ifdef DEBUG
    fprintf(stderr,"In service register callback %x\n", &sdRef);
    fprintf(stderr,"  callback = %x\n", func);
    fprintf(stderr,"  userdata = %x\n", userdata);
    #endif
    arglist = Py_BuildValue("(OiisssO)",sdRef,flags,errorCode,name,
                                        regtype,domain,userdata);
    if (PyErr_Occurred() != NULL){
	    PyErr_Print();
    } 		
    result = PyObject_CallObject(func, arglist);
    if (PyErr_Occurred() != NULL){
	    PyErr_Print();
    } 		
    Py_XDECREF(result);
    Py_DECREF(arglist);
    #ifdef DEBUG
    fprintf(stderr,"Exiting service register callback\n");
    #endif
    }

DNSServiceErrorType
pyDNSServiceRegister
    (
    DNSServiceRef                       *sdRef,
    DNSServiceFlags                     flags,
    uint32_t                            interfaceIndex,
    const char                          *name,         /* may be NULL */
    const char                          *regtype,
    const char                          *domain,       /* may be NULL */
    const char                          *host,         /* may be NULL */
    uint16_t                            port,
    uint16_t                            txtLen,
    const char                          *txtRecord,    /* may be NULL */
    PyObject                            *callback,     /* may be NULL */
    PyObject                            *context
    )
    {
    DNSServiceErrorType ret;
    CallbackStruct *cbStruct = (CallbackStruct*)malloc(sizeof(CallbackStruct));
    Py_XINCREF(callback);
    Py_XINCREF(context);
    cbStruct->func = callback;
    cbStruct->userdata = context;
    ret = DNSServiceRegister( sdRef, flags, interfaceIndex,
                              name, regtype, domain,
                              host, port, txtLen, txtRecord,
                              (DNSServiceRegisterReply)service_register_callback,
                              (void *)cbStruct); 
    return ret;
    }


static void DNSSD_API
service_browse_callback
    (
    DNSServiceRef                       sdRef,
    DNSServiceFlags                     flags,
    uint32_t                            interfaceIndex,
    DNSServiceErrorType                 errorCode,
    const char                          *serviceName,
    const char                          *regtype,
    const char                          *replyDomain,
    void                                *context
    )
    {
    PyObject *func ,*userdata, *arglist, *result;
    
    CallbackStruct *cbStruct = (CallbackStruct *)context;
    func = (PyObject*)cbStruct->func;
    userdata = (PyObject*)cbStruct->userdata;
        
    #ifdef DEBUG
    fprintf(stderr,"In service browse callback %x\n", &sdRef);
    #endif
    arglist = Py_BuildValue("(OiiisssO)",sdRef,flags,interfaceIndex,
                                       errorCode,serviceName,regtype,
                                       replyDomain,userdata);
    if (PyErr_Occurred() != NULL){
	    PyErr_Print();
    } 		
    result = PyObject_CallObject(func, arglist);
    if (PyErr_Occurred() != NULL){
	    PyErr_Print();
    } 		
    Py_XDECREF(result);
    Py_DECREF(arglist);
    #ifdef DEBUG
    fprintf(stderr,"Exiting service browse callback\n");
    #endif
    }    


DNSServiceErrorType 
pyDNSServiceBrowse
    (
    DNSServiceRef                       *sdRef,
    DNSServiceFlags                     flags,
    uint32_t                            interfaceIndex,
    const char                          *regtype,
    const char                          *domain,    /* may be NULL */
    PyObject                            *callback,
    PyObject                            *context
    )
    {
    DNSServiceErrorType ret;
    CallbackStruct *cbStruct = (CallbackStruct*)malloc(sizeof(CallbackStruct));
    Py_XINCREF(callback);
    Py_XINCREF(context);
    cbStruct->func = callback;
    cbStruct->userdata = context;
    ret = DNSServiceBrowse( sdRef, flags, interfaceIndex,
                            regtype, domain,
                            (DNSServiceBrowseReply)service_browse_callback,
                            (void *)cbStruct);
    return ret;
    }


static void DNSSD_API
service_resolve_callback
    (
    DNSServiceRef                       sdRef,
    DNSServiceFlags                     flags,
    uint32_t                            interfaceIndex,
    DNSServiceErrorType                 errorCode,
    const char                          *fullname,
    const char                          *hosttarget,
    uint16_t                            port,
    uint16_t                            txtLen,
    const char                          *txtRecord,
    void                                *context
    )
    {
    PyObject *func ,*userdata, *arglist, *result;
    CallbackStruct *cbStruct = (CallbackStruct *)context;
    func = (PyObject*)cbStruct->func;
    userdata = (PyObject*)cbStruct->userdata;
    
    #ifdef DEBUG
    fprintf(stderr,"In service resolve callback\n");
    #endif
    arglist = Py_BuildValue("(Oiiissiis#O)",sdRef,flags,interfaceIndex,
                                       errorCode,fullname,hosttarget,
                                       port,txtLen,txtRecord,txtLen,userdata);
    if (PyErr_Occurred() != NULL){
	    PyErr_Print();
    } 		
    result = PyObject_CallObject(func, arglist);
    if (PyErr_Occurred() != NULL){
	    PyErr_Print();
    } 		
    Py_XDECREF(result);
    Py_DECREF(arglist);
    #ifdef DEBUG
    fprintf(stderr,"Exiting service resolve callback\n");
    #endif
    }
    
DNSServiceErrorType 
pyDNSServiceResolve
    (
    DNSServiceRef                       *sdRef,
    DNSServiceFlags                     flags,
    uint32_t                            interfaceIndex,
    const char                          *name,
    const char                          *regtype,
    const char                          *domain,
    PyObject                            *callback,
    PyObject                            *context
    )
    {
    DNSServiceErrorType ret;
    CallbackStruct *cbStruct = (CallbackStruct*)malloc(sizeof(CallbackStruct));
    Py_XINCREF(callback);
    Py_XINCREF(context);
    cbStruct->func = callback;
    cbStruct->userdata = context;
    ret = DNSServiceResolve( sdRef, flags, interfaceIndex,
                             name, regtype, domain,
                             (DNSServiceResolveReply)service_resolve_callback,
                             (void *)cbStruct);
    return ret;
    }

static void DNSSD_API
service_query_callback
    (
    DNSServiceRef                       sdRef,
    DNSServiceFlags                     flags,
    uint32_t                            interfaceIndex,
    DNSServiceErrorType                 errorCode,
    const char                          *fullname,
    uint16_t                            rrtype,
    uint16_t                            rrclass,
    uint16_t                            rdlen,
    const void                          *rdata,
    uint32_t                            ttl,
    void                                *context
    )
    {
    PyObject *func ,*userdata, *arglist, *result;
    CallbackStruct *cbStruct = (CallbackStruct *)context;
    func = (PyObject*)cbStruct->func;
    userdata = (PyObject*)cbStruct->userdata;
    
    #ifdef DEBUG
    fprintf(stderr,"In service query callback\n");
    #endif
    arglist = Py_BuildValue("(Oiiisiiis#iO)",sdRef,flags,interfaceIndex,
                                       errorCode,fullname,rrtype,
                                       rrclass,rdlen,(char*)rdata,rdlen,
                                       ttl,userdata);
    if (PyErr_Occurred() != NULL){
	    PyErr_Print();
    } 		
    result = PyObject_CallObject(func, arglist);
    if (PyErr_Occurred() != NULL){
	    PyErr_Print();
    } 		
    Py_XDECREF(result);
    Py_DECREF(arglist);
    #ifdef DEBUG
    fprintf(stderr,"Exiting service query callback\n");
    #endif
    }
    
DNSServiceErrorType 
pyDNSServiceQueryRecord
    (
    DNSServiceRef   *sdRef,
    DNSServiceFlags flags,
    uint32_t        interfaceIndex,
    const char      *fullname,
    uint16_t        rrtype,
    uint16_t        rrclass,
    PyObject        *callback,
    PyObject        *context
    )
    {
    DNSServiceErrorType ret;
    CallbackStruct *cbStruct = (CallbackStruct*)malloc(sizeof(CallbackStruct));
    Py_XINCREF(callback);
    Py_XINCREF(context);
    cbStruct->func = callback;
    cbStruct->userdata = context;
    ret = DNSServiceQueryRecord( sdRef, flags, interfaceIndex,
                                 fullname, rrtype, rrclass,
                                 (DNSServiceQueryRecordReply)service_query_callback,
                                 (void *)cbStruct);
    return ret;
    }
    


%}

extern enum
    {
    kDNSServiceFlagsMoreComing         ,
    kDNSServiceFlagsAdd                ,
    kDNSServiceFlagsDefault            ,
    kDNSServiceFlagsNoAutoRename       ,
    kDNSServiceFlagsShared             ,
    kDNSServiceFlagsUnique             ,
    kDNSServiceFlagsBrowseDomains      ,
    kDNSServiceFlagsRegistrationDomains,
    };

extern enum
    {
    kDNSServiceErr_NoError                            ,
    kDNSServiceErr_Unknown                            ,
    kDNSServiceErr_NoSuchName                         ,
    kDNSServiceErr_NoMemory                           ,
    kDNSServiceErr_BadParam                           ,
    kDNSServiceErr_BadReference                       ,
    kDNSServiceErr_BadState                           ,
    kDNSServiceErr_BadFlags                           ,
    kDNSServiceErr_Unsupported                        ,
    kDNSServiceErr_NotInitialized                     ,
    kDNSServiceErr_AlreadyRegistered                  ,
    kDNSServiceErr_NameConflict                       ,
    kDNSServiceErr_Invalid                            ,
    kDNSServiceErr_Incompatible                       ,
    /* mDNS Error codes are in the range
     * FFFE FF00 (-65792) to FFFE FFFF (-65537) */
    };

DNSServiceErrorType DNSServiceProcessResult(DNSServiceRef sdRef);
void DNSServiceRefDeallocate(DNSServiceRef sdRef);
int DNSServiceRefSockFD(DNSServiceRef sdRef);

DNSServiceRef * AllocateDNSServiceRef();

